/*
 **
 ** SensorsUpdate.java - Update sesnors data to be served through embedded HTTP server
 **
 ** Copyright (C) 2019 Elphel, Inc.
 **
 ** -----------------------------------------------------------------------------**
 **
 **  SensorsUpdate.java is free software: you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
 **  the Free Software Foundation, either version 3 of the License, or
 **  (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ** -----------------------------------------------------------------------------**
 **
 */



package com.elphel.sensorserver;

import android.app.Activity;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.util.Log;

public class SensorsUpdate implements SensorEventListener{
    private static String  TAG = "SENSORS";

	private final Activity activity;
    private SensorManager sensorManager;
    private final float[] accelerometerReading = new float[3];
    private final float[] magnetometerReading =  new float[3];

    private final float[] rotationMatrix =       new float[9];
//    private final float[] orientationAngles =    new float[3];
//    private final float[] mMagnetometerReading = new float[3];
    private final float[] mOrientationAngles =   new float[3];


	public SensorsUpdate (Activity activity) {
		this.activity = activity;

	}

	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onSensorChanged(SensorEvent event) {
//		Log.e(TAG,"onSensorChanged:" + event.toString());

        if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
            System.arraycopy(event.values, 0, accelerometerReading,
                0, accelerometerReading.length);
          } else if (event.sensor.getType() == Sensor.TYPE_MAGNETIC_FIELD) {
              System.arraycopy(event.values, 0, magnetometerReading,
                  0, magnetometerReading.length);
          }
	}

	public void start() {
		sensorManager = (SensorManager) activity.getSystemService(Context.SENSOR_SERVICE);
		Log.i(TAG,"start()");

	}

	public void resume() {
        Sensor accelerometer = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
        if (accelerometer != null) {
            sensorManager.registerListener(this, accelerometer,
                SensorManager.SENSOR_DELAY_NORMAL, SensorManager.SENSOR_DELAY_UI);
    		Log.i(TAG,"accelerometer is NOT null");
        } else {
    		Log.e(TAG,"accelerometer is null");
        }
        Sensor magneticField = sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);
        if (magneticField != null) {
            sensorManager.registerListener(this, magneticField,
                SensorManager.SENSOR_DELAY_NORMAL, SensorManager.SENSOR_DELAY_UI);
    		Log.i(TAG,"magneticField is NOT null");
        } else {
    		Log.e(TAG,"magneticField is null");
        }
		Log.e(TAG,"resume() done");
	}

	public void pause() {
		sensorManager.unregisterListener(this);
	}

    private void updateOrientationAngles() {
        // Update rotation matrix, which is needed to update orientation angles.
		Log.e(TAG,"updateOrientationAngles()");
        SensorManager.getRotationMatrix(rotationMatrix, null,
//'            accelerometerReading, mMagnetometerReading);
               accelerometerReading, magnetometerReading);

        // "mRotationMatrix" now has up-to-date information.

        SensorManager.getOrientation(rotationMatrix, mOrientationAngles);

        // "mOrientationAngles" now has up-to-date information.
    }

    public float [] getOrientation() {
    	updateOrientationAngles(); // todo: synchronize
    	return mOrientationAngles;
    }

}
